#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-file-transfer-agents-config v. 2.2.0
#
# Post-installation script for configuring the gLite File Transfer Agents
# Robert Harakaly  < robert.harakaly@cern.ch >
#
# Version info: $Id: glite-file-transfer-agents-config.py,v 1.5 2006/03/21 14:59:35 flammer Exp $
#
# Usage: python glite-file-transfer-agents-config [-c|-v|-h|--start|--stop|--status|--configure--help]
#        -c          print configuration
#        -v          print version
#        -h,--help   print usage info
#        --start     start the service
#        --stop      start the service
#        --status    get the service status (o=running, 1=idle) 
#        --configure configure the service and database (default)
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################

import os,string,socket,pwd,time
import sys
import getopt
import commands

from gLiteInstallerLib import gLib
from gLiteInstallerLib import ConfigParams
from gliteRgmaServicetool import gliteRgmaServicetoolInstance
from gliteRgmaServicetool import gliteRgmaServicetool
from serviceDiscovery import ServiceDiscovery
from gLiteInstallerLib import UltraPrettyPrint


# Set global variables here 
global params                         # all config values from the XML file
global verbose
global glib
global rgmaServicetool

class glite_data_transfer_agents:    
    def __init__(self, params):
        self.params = params
        self.verbose = 0
        self.version = "2.2.0"
        self.name = "glite-file-transfer-agents"
        self.friendly_name = "gLite File Transfer Agents"

    #-------------------------------------------------------------------------------
    # Banner 
    #-------------------------------------------------------------------------------

    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #-------------------------------------------------------------------------------
    # Copyright 
    #-------------------------------------------------------------------------------

    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'

    #-------------------------------------------------------------------------------
    # Version
    #-------------------------------------------------------------------------------

    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    
    #-------------------------------------------------------------------------------
    # Usage
    #-------------------------------------------------------------------------------

    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc.config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION...]""" % (self.name, os.environ['GLITE_LOCATION'], \
        os.environ['GLITE_LOCATION'], self.name, self.name)

        print '    -c, --checkconf                       print the service configuration'
        print '    -v, --version                         print the version of the configuration script'
        print '    -h, --help                            print this usage information'
        print '    --configure                           configure the service and database (default)'
        print '    --remove                              remove the service and database schemas'
        print '    --start                               start the service'
        print '    --stop                                stop the service'
        print '    --status                              get the service status (o=running, 1=idle)'
        print '    --siteconfig=<site config url>        use site configuration file'
        print '\n\n    Per instance commands. Commands marked (L) work only with the local configuration files'
        print '    --instance add <instance_name> <service_type> '
        print """
                     <service_type> one of:            
                                 transfer-channel-agent-urlcopy
                                 transfer-channel-agent-srmcopy
                                 transfer-vo-agent-fps
                                 transfer-vo-agent-fts    """
        print '    --instance <command> <instance_name>'
        print """
                     <command> one of:
                                 configure
                                 start
                                 stop
                                 remove
                                 status
        """
        print '\n'

    def configure(self):
        if os.path.exists("%s/etc/services.xml" % os.environ['GLITE_LOCATION']):
            glib.printInfoMessage("Creating backup of the services.xml file")
            os.system("cp %s/etc/services.xml %s/etc/services.xml.back" \
                % (os.environ['GLITE_LOCATION'], os.environ['GLITE_LOCATION']))
        if params['file-transfer.DBBackend'] == 'oracle':
            #-------------------------------------------------------------------------------
            # Check Oracle instanclient libs
            #-------------------------------------------------------------------------------
            if params.has_key('file-transfer.oracle-instantclient.location'):
                if params['file-transfer.oracle-instantclient.location'] != "":
                    if not os.path.exists(params['file-transfer.oracle-instantclient.location']):
                        glib.printErrorMessage("The Oracle InstantClient cannot be found at the specified location")
                        glib.printErrorMessage(params['file-transfer.oracle-instantclient.location'])
                        glib.printErrorMessage("Configuration cannot continue")
                        return 1
            
        #-------------------------------------------------------------------------------
        # Installs the Security Utilities
        #--------------------------------------------------------
        
        if os.system("python %s/glite-security-utils-config.py --subservice" % glib.getScriptPath()):
            print "\nConfiguring gLite Security Utilities               ",
            glib.printFailedMessage()
            return 1

        print "\nConfiguring gLite Security Utilities                               ",
        glib.printOkMessage()
            
        #-----------------------------------------------------------------------------------------
        # Create the user account
        #-----------------------------------------------------------------------------------------

        print 'Creating and configuring the %s user account' % self.friendly_name

        if ((params.has_key('transfer.agent.user.uid') and not params.has_key('transfer-agent.group.gid')) or
            (params.has_key('transfer-agent.group.gid') and not params.has_key('transfer.agent.user.uid'))):
            print "Both the uid and gid must be set or neither - please set them and rerun"
            return 1

        if not params.has_key('transfer.agent.user.uid'):
            params['transfer.agent.user.uid'] = "0"
        if not params.has_key('transfer-agent.group.gid'):
            params['transfer-agent.group.gid'] = "0"

        if params['transfer.agent.user.uid'] == "":
            params['transfer.agent.user.uid'] = "0"

        if params['transfer-agent.group.gid'] == "":
            params['transfer-agent.group.gid'] = "0"

        (uid,gid) = glib.add_user(params['transfer-agent.username'], params['transfer-agent.groupname'],\
            params['transfer.agent.user.uid'], params['transfer-agent.group.gid'])
        if uid == -1:
            return 1

        print 'User account created                                    ',
        glib.printOkMessage()

        uid,gid = glib.get_user_info(params['transfer-agent.username'])
        
        # Check home directory, create if not existing
        if self.verbose:
            print "Checking user home directory %s" % pwd.getpwnam(params['transfer-agent.username'])[5]
        glib.check_dir("%s" % pwd.getpwnam(params['transfer-agent.username'])[5], 0700, uid, gid)
    
        # Reset environment, just in case
        glib.setUserEnv(params['transfer-agent.username'])

        # Verify and create service directories
        if self.verbose:
            print "Verify directory permissions"
        glib.check_dir(os.environ['GLITE_LOCATION_VAR'], 0755, uid, gid)
        glib.check_dir(os.environ['GLITE_LOCATION_LOG'], 0755, uid, gid)
        glib.check_dir(os.environ['GLITE_LOCATION_TMP'], 0755, uid, gid)
        glib.check_dir("%s/lock" % os.environ['GLITE_LOCATION_VAR'], 0755, uid, gid)
        glib.check_dir("%s/tmp" % os.environ['GLITE_LOCATION_VAR'], 0755, uid, gid)
                
        #-----------------------------------------------------------------------------------------
        # Copy the host certificates to the user account
        #-----------------------------------------------------------------------------------------
        
        if params['service.certificates.type'] == 'host':
            
            print "Copying the host certificate to the %s user account" % self.friendly_name
    
            uid,gid = glib.get_user_info(params['transfer-agent.username'])
            
            # Check certificate directory, create if not existing
            if not params['fta.certificate.path'].startswith("/"):
                params['fta.certificate.path'] = "%s/%s" \
                    %(pwd.getpwnam(params['transfer-agent.username'])[5], params['fta.certificate.path'])

                if self.verbose:
                    print "Checking user certificates directory %s" % params['fta.certificate.path']
                glib.check_dir("%s" % params['fta.certificate.path'], 0700, uid, gid)
                
            # Copy certificates
            if self.verbose:
                print "Copying host certificates to I/O Server user certificates directory"
            os.system("cp %s %s/glite-file-transfer-agents-cert.pem" \
                % (params['host.certificate.file'],  params['fta.certificate.path']))
            os.system("cp %s %s/glite-file-transfer-agents-key.pem" \
                % (params['host.key.file'], params['fta.certificate.path']))
            os.chown("%s/glite-file-transfer-agents-cert.pem" % params['fta.certificate.path'], uid, gid)
            os.chown("%s/glite-file-transfer-agents-key.pem" % params['fta.certificate.path'], uid, gid)
            os.chmod("%s/glite-file-transfer-agents-cert.pem" % params['fta.certificate.path'], 0644)
            os.chmod("%s/glite-file-transfer-agents-key.pem" % params['fta.certificate.path'], 0400)
       
            if (os.path.exists('%s/glite-file-transfer-agents-key.pem' % params['fta.certificate.path'])) and \
                    (os.path.exists('%s/glite-file-transfer-agents-cert.pem' % params['fta.certificate.path'])):
                print "Copied host certificates to %s            " % params['fta.certificate.path'],
                glib.printOkMessage()
            else:
                print "An error occurred while copying the host certificates to %s                " \
                    % params['fta.certificate.path'],  glib.printFailedMessage()
                return 1
    
            # Define service certificates location
            params['service.certificate.file'] = "%s/glite-file-transfer-agents-cert.pem" % params['fta.certificate.path'];
            params['service.key.file'] = "%s/glite-file-transfer-agents-key.pem" % params['fta.certificate.path'];
                    
        elif params['service.certificates.type'] == 'service':
    
            if (os.path.exists(params['service.certificate.file'])) and \
                    (os.path.exists(params['service.key.file'])):

                # Check permissions
                uid,gid = glib.get_user_info(params['transfer-agent.username'])
                glib.check_dir("%s" % os.path.dirname(params['service.certificate.file']),0755, uid, gid)
                os.chown(params['service.certificate.file'], uid, gid)
                os.chmod(params['service.certificate.file'],0644)
                glib.check_dir("%s" % os.path.dirname(params['service.key.file']),0755, uid, gid)
                os.chown(params['service.key.file'], uid, gid)
                os.chmod(params['service.key.file'],0400)
    
                print "Using certificates %s and %s            " % (params['service.certificate.file'],\
                    params['service.key.file']),
                glib.printOkMessage()
            else:
                glib.printWarningMessage("Service certificates %s and %s not found" % (params['service.certificate.file'],\
                    params['service.key.file']))
                print "Please install the certificates with correct ownership before starting the %s" % self.friendly_name
                error_level = 2

        else:
            print "The type of certificates to use for this service is not specified. Cannot continue    ",
            glib.printFailedMessage()
            return 1
            
        # SSL configuration
        #glib.export('X509_USER_CERT',params['service.certificate.file']);
        #glib.export('X509_USER_KEY',params['service.key.file']);
 
        # Instance naming MUST be transfer-...
        instances = glib.getInstances("transfer-")
        try:
            if len(instances) == 0:
                print "No tansfer agent instance defined quitting"
                return 1
            else:
                for name in instances.keys():
                    try:
                        if self.configureInstance(name) != 0:
                            glib.printErrorMessage("Could not configure agent instance %s" \
                                % name)
                            return 1
                    except LookupError, msg:
                        glib.printErrorMessage("%s - Cannot continue" % msg)
                        return 1
        except ValueError, msg:
            glib.printErrorMessage("%s - Cannot continue" % msg)
            return 1


        # add log rotate ...
        logrotateString = "/var/log/glite/glite-transfer-*.log {\n\
    daily\n\
    missingok\n\
    compress\n\
    delaycompress\n\
    rotate 15\n\
    sharedscripts\n\
    postrotate\n\
         (. /etc/profile ; . /etc/glite/profile.d/glite_setenv.sh ; \
         %s/etc/config/scripts/glite-file-transfer-agents-config.py \
         --restart > /dev/null 2>&1 || true)\n\
    endscript\n}\n" % os.environ['GLITE_LOCATION']
        logrotate = open("/etc/logrotate.d/file-transfer-agents", "w")
        logrotate.write(logrotateString)
        logrotate.close()

#        log_rotate_file = "%s/etc/logrotate.d/glite-file-transfer-agents" % os.environ['GLITE_LOCATION']
#        if os.path.exists(log_rotate_file):
#           os.symlink(log_rotate_file, "/etc/logrotate.d/glite-file-transfer-agents")  

        # ------------------------------------------------------------
        # configure service discovery
        # ------------------------------------------------------------
        errorcode =    serviceDiscovery.configure(params,glib)
        if errorcode:
            return 1
    
        #-------------------------------------------------------------------
        # RGMA servicetool: configure servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
            
            instances = glib.getInstances("transfer-")
            if len(instances) > 0:
                for name in instances.keys():
    
                    # Instantiate the rgma-servicetool class
                    rgmaServicetool = gliteRgmaServicetool()
                    rgmaServicetool.verbose = self.verbose
                    
                    # Create FTS instance
                    serviceId = "%s_%s_%s" % (glib.fq_hostname, name, params['transferAgents.serviceType'])
                    servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
                    
                    # set params
                    servicetoolInstance.setServiceName('File Transfer Agent %s at %s' % (name, glib.fq_hostname))
                    servicetoolInstance.setServiceType(params['transferAgents.serviceType'])
                    servicetoolInstance.setServiceVersion(self.version)
                    servicetoolInstance.setStatusScript(params['transferAgents.statusScript'])
                    servicetoolInstance.setEndpoint(params['transferAgents.endpoint'])
                    
                    # add instance to the gLite configuration
                    if servicetoolInstance.add() == 1:
                        return 1
                    
            # Configure servicetool
            if rgmaServicetool.configure(glib):
                # error in configuring servicetool
                return 1
            
        return 0
        
    def configureInstance(self, name='default'):
        
        instanceParams = {}
        instanceParams = glib.getInstanceParams(name)
    
        instanceName = name.split(':')[1]
        instanceType = name.split(':')[0]
        
        outputBase = '%s/etc/glite-%s-%s' % (os.environ['GLITE_LOCATION'],\
            instanceType[0:instanceType.rfind('-')], instanceName)
        
        #print "deployment type: %s " % instanceType
        #print "Configure transfer-agent instance: %s" % name.split(':')[1]
       
        properties_file = "%s/etc/config/glite-%s-%s.properties" % (os.environ['GLITE_LOCATION'], instanceType, instanceName)

        # Definition of the value of the 'transfer-agent.log.Filename' parameter
        instanceParams['transfer-agent.log.Filename'] = '%s/glite-%s-%s.log' \
            %(instanceParams['transfer-agent.logdir'], instanceType, instanceName)
        instanceParams[instanceParams['transfer-agent.name.prefix'] + '.Name'] = instanceName
        
        f = open(properties_file,'w')
        for param in instanceParams.keys():
            if param.find('transfer-agent.log.') > -1:
                f.write("log.%s=%s\n" %(param.split(".")[2], instanceParams[param]))
            else:
                if param.find('transfer-agent-') > -1:
                    f.write("%s=%s\n" %(param, instanceParams[param]))
                if param.find('transfer-vo') > -1:
                    f.write("%s=%s\n" %(param, instanceParams[param]))    
                if param.find('transfer-channel') > -1:
                    f.write("%s=%s\n" %(param, instanceParams[param]))            
        f.close()
        #-----------------------------------------------------------------------------------------
        # Run the configuration tool
        #-----------------------------------------------------------------------------------------
        print "Running the configuration tool: glite_data_config_generator"

        try:
            config_generator = '%s/bin/glite_data_config_generator' % os.environ['GLITE_LOCATION']
            template = '%s/share/config/glite-data-transfer-agents/glite-%s-%s.config.xml' \
                    % (os.environ['GLITE_LOCATION'], instanceType, params['file-transfer.DBBackend'])
            #print "template %s" %template    
            output = '%s.properties.xml' % outputBase
            #print "output %s" %output
            log = '%s.log-properties' % outputBase
            cmd = "%s -f %s -o %s -l %s  -p %s"  % (config_generator,template,output,log,properties_file)
            if self.verbose > 0 :
                print "Executing command %s" %cmd    
            if os.path.exists(output):
                os.remove(output)

            print "Configuring %s                 " % instanceName
            if os.system(cmd) == 0:
                glib.printOkMessage()
            else:
                glib.printFailedMessage()

            # Remove the properties file
            os.remove(properties_file)

        except Exception, msg:

            glib.printErrorMessage("An exception occurred while creating the agent configuration files")
            glib.printErrorMessage("(%s)" % msg)
            return 1
        
        return 0

    def remove(self):
        ################################
        #    Remove all instances
        ################################

        instances = glib.getInstances("transfer-")
        if len(instances) == 0:
            self.removeInstance()
        else:
            for name in instances.keys():
                self.removeInstance(name)
        if os.path.exists("/etc/logrotate.d/file-transfer-agents"):
            os.remove("/etc/logrotate.d/file-transfer-agents")
        return 0
    
    def removeInstance(self, name='default'):
        
        instanceParams = glib.getInstanceParams(name)
        instanceType, instanceName = name.split(':')
        
        outputBase = '%s/etc/glite-%s-%s' % (os.environ['GLITE_LOCATION'],\
            instanceType[0:instanceType.rfind('-')], instanceName)
        output = '%s.properties.xml' % outputBase
        log = '%s.log-properties' % outputBase
        print "Removing transfer-agent instance: %s" % instanceName    
            
        if os.path.exists(output):
            os.remove(output)
        if os.path.exists(log):
            os.remove(log)
            
        return 0
    
    def start(self):
        
        error_level = 0
        
        if params['file-transfer.DBBackend'] == 'oracle':
            # Check Oracle instanclient libs
            if params.has_key('file-transfer.oracle-instantclient.location'):
                if params['file-transfer.oracle-instantclient.location'] != "":
                    if not os.path.exists(params['file-transfer.oracle-instantclient.location']):
                        glib.printErrorMessage("The Oracle InstantClient cannot be found at the specified location")
                        glib.printErrorMessage(params['file-transfer.oracle-instantclient.location'])
                        glib.printErrorMessage("Cannot start the %s" % self.friendly_name)
                        return 1

        instances = glib.getInstances("transfer-")
        try:
            if len(instances) == 0:
                try:
                    error_level = self.startInstance()
                except LookupError, msg:
                    glib.printErrorMessage("%s - Cannot continue" % msg)
                    return 1
            else:
                for name in instances.keys():
                    try:
                        error_level = self.startInstance(name)
                    except LookupError, msg:
                        glib.printErrorMessage("%s - Cannot continue" % msg)
                        return 1
        except ValueError, msg:
            glib.printErrorMessage("%s - Cannot continue" % msg)
            return 1
                
        if params["watchdog.enable"] == 'true':
            self.addWatchdog()
        
        #-------------------------------------------------------------------
        # Start Servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
        
            errorcode = rgmaServicetool.start()
            if (errorcode != 0):
                return 1
        
        return error_level
        
    def stop(self):

        error_level = 0
        
        if params["watchdog.enable"] == 'true':
            self.stopWatchdog()        

        instances = glib.getInstances("transfer-")
        try:
            if len(instances) == 0:
                if self.stopInstance() != 0:
                    error_level = 1
            else:
                for name in instances.keys():
                    try:
                        if self.stopInstance(name) != 0:
                            glib.printErrorMessage("Could not stop agent instance %s" \
                                % name)
                            error_level = 1
                    except LookupError, msg:
                        glib.printErrorMessage("%s" % msg)
                        error_level = 1
                        
        except ValueError, msg:
            glib.printErrorMessage("%s" % msg)
            error_level = 1

        #-------------------------------------------------------------------
        # Servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
            
            if rgmaServicetool.stop():
                error_level = 1
    
        return error_level
        
    def startInstance(self, name='default'):
        
        self.stopInstance(name)
        instanceParams = {}
        instanceParams = glib.getInstanceParams(name)
        type = name.split(':')[0]
        instanceType = type[type.find('-') + 1:type.rfind('-')]
        instanceName = name.split(':')[1]
        print "Starting %s                " % instanceName    
        agentProcess = 'glite-transfer-%s-%s' % (instanceType, instanceName)
        startAgentCommand = "/bin/su - %s -c '%s/sbin/glite-data-config-service-wrapper glite-file-transfer-agents %s start'" \
            %(instanceParams['transfer-agent.username'], os.environ['GLITE_LOCATION'], agentProcess)
        
        ret = os.system(startAgentCommand)
        if ret == 1:
            glib.printFailedMessage()
            return 1
        else:
            glib.printOkMessage()
            
        return 0

    def stopInstance(self, name='default'):
        
        instanceParams = {}
        instanceParams = glib.getInstanceParams(name)
    
        type = name.split(':')[0]
        instanceType = type[type.find('-') + 1:type.rfind('-')]
        instanceName = name.split(':')[1]
        print "Stopping %s                " % instanceName
        agentProcess = 'glite-transfer-%s-%s' % (instanceType, instanceName)
        stopAgentCommand = "/bin/su -l %s -c '%s/bin/glite_data_config_service %s stop'" \
            %(instanceParams['transfer-agent.username'], os.environ['GLITE_LOCATION'], agentProcess)    
            
        ret = os.system(stopAgentCommand)
        if ret == 1:
            glib.printFailedMessage()
            return 1
        else:
            glib.printOkMessage()
            
        return 0
        
    def status(self):

        error_level = 0
        
        instances = glib.getInstances("transfer-")
        try:
            if len(instances) == 0:
                self.statusInstance()
            else:
                for name in instances.keys():
                    try:
                        self.statusInstance(name)
                    except LookupError, msg:
                        glib.printErrorMessage("%s - Unable to detect agent status" % msg)
                        error_level = 1        
        except ValueError, msg:
            glib.printErrorMessage("%s - Unable to detect agent status" % msg)        
            error_level = 1
                

        #-------------------------------------------------------------------
        # Servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
            
            if rgmaServicetool.status() != 0:
                error_level = 1
    
        return error_level
        
    def statusInstance(self, name='default'):
        instanceParams = {}
        instanceParams = glib.getInstanceParams(name)

        type = name.split(':')[0]
        instanceType = type[type.find('-') + 1:type.rfind('-')]
        instanceName = name.split(':')[1]
        agentProcess = 'glite-transfer-%s-%s' % (instanceType, instanceName)
        statusAgentCommand = "/bin/su -l %s -c '%s/bin/glite_data_config_service %s status'" \
            %(instanceParams['transfer-agent.username'], os.environ['GLITE_LOCATION'], agentProcess)
            
        return os.system(statusAgentCommand)

    def addWatchdog(self):

        instances = glib.getInstances()
        if len(instances) == 0:
            #self.configureInstance()
            pass
        else:
            for name in instances.keys():

                instanceParams = glib.getInstanceParams(name)
                type = name.split(':')[0]
                instanceType = type[type.find('-') + 1:type.rfind('-')]
                instanceName = name.split(':')[1]
                agentProcess = 'glite-transfer-%s-%s' % (instanceType, instanceName)
                watchdogCheck = "%s/bin/glite_data_config_service glite-transfer-%s-%s status" \
                    %(os.environ['GLITE_LOCATION'],instanceType, instanceName)
                watchdogResult = "Service glite-transfer-%s-%s status is running" %(instanceType, instanceName)
                glib.addWatchdog(instanceName, watchdogCheck, watchdogResult,\
                    'python %s/etc/config/scripts/glite-file-transfer-agents-config.py --start' \
                    % os.environ['GLITE_LOCATION'],"")

    def stopWatchdog(self):

        instances = glib.getInstances("transfer-")
        if len(instances) == 0:
            #self.configureInstance()
            pass
        else:
            for name in instances.keys():
                print "Removing watchdog configuration for instance: %s" % name.split(':')[1]
                watchDog_xml = "%s/etc/watchdog.d/glite-transfer-agents-%s.xml" \
                        % (params['GLITE_LOCATION'], name.split(':')[1])
                if os.path.exists(watchDog_xml):
                        os.remove(watchDog_xml)
         

def replaceTags(line, params, prefix):
    if line.find("@") > -1:
        tags = line.split("@")
        i = 1
        while i < len(tags):
            if params.has_key(prefix + "." + tags[i]):
                paramName = "%s.%s" % (prefix, tags[i])
                print("Replace @%s@ by %s" % (tags[i], params[paramName]))
                line = line.replace("@%s@" % tags[i], params[paramName])
            else:
                print("Replace @%s@ by empty string" %(tags[i]))
                line = line.replace("@%s@" % tags[i], "")
            i += 2
    return line
        
#-------------------------------------------------------------------------------
# Default param values
#-------------------------------------------------------------------------------

def loadDefaults(params):

    params['GLITE_LOCATION'] = "/opt/glite"
    
    params['transferAgents.serviceType'] = 'org.glite.FileTransferAgents'
    params['transferAgents.statusScript'] = 'not available'
    params['transferAgents.endpoint'] = 'not available'
    
#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------

def set_env():
    glib.export('GLITE_LOCATION')
    glib.export('GLITE_LOCATION_VAR')
    glib.export('GLITE_LOCATION_LOG')
    glib.export('GLITE_LOCATION_TMP')
    glib.export('GLOBUS_LOCATION')
    glib.export('GPT_LOCATION')
    glib.export('LD_LIBRARY_PATH')
    
    glib.check_dir(os.environ['GLITE_LOCATION_VAR'])
    glib.check_dir(os.environ['GLITE_LOCATION_LOG'], 0777)
    glib.check_dir(os.environ['GLITE_LOCATION_TMP'])
    
    glib.export("JAVA_HOME")

    glib.addEnvPath("PATH", "%s/bin" % os.environ["JAVA_HOME"])
    glib.addEnvPath("PATH", "%s/externals/bin" % os.environ["GLITE_LOCATION"])
    glib.addEnvPath("PATH", "%s/bin" % os.environ["GLITE_LOCATION"])

    glib.addEnvPath("LD_LIBRARY_PATH", "/usr/local/lib")
    glib.addEnvPath("LD_LIBRARY_PATH", "%s/lib" % os.environ['GLOBUS_LOCATION'])
    glib.addEnvPath("LD_LIBRARY_PATH", "%s/externals/lib" % os.environ["GLITE_LOCATION"])
    glib.addEnvPath("LD_LIBRARY_PATH", "%s/lib" % os.environ["GLITE_LOCATION"])
    if params['file-transfer.DBBackend'] == 'oracle':
        glib.addEnvPath("LD_LIBRARY_PATH", "%s/lib/" % params['file-transfer.oracle-instantclient.location'])
    
    # Perl
    glib.addEnvPath("PERL5LIB", "%s/lib/perl:%s/lib/perl5" % (os.environ['GPT_LOCATION'],\
        os.environ['GLITE_LOCATION']))

    # Fix X509_USER_{CERT|KEY} problem in setenv file
    if os.path.exists(params['installer.export.filename']):
        infile = open(params['installer.export.filename'],'r')
        outfile = open(params['installer.export.filename']+'.1','w')
        for line in infile:
            if line.find('X509_USER_CERT') == -1 and \
                line.find('X509_USER_KEY') == -1:
                outfile.write(line)
                
        outfile.close()
        infile.close()
        os.rename(params['installer.export.filename']+'.1',params['installer.export.filename'])

def findFullInstanceName(name):
    instances = glib.getInstances("transfer-").keys()
    for i in instances:
        if i.split(':')[1] == name:
            name = i
            break
    return name

def addInstanceToConfig(name, service):
    config_file = "%s/../glite-file-transfer-agents.cfg.xml" % glib.getScriptPath()
    if os.path.exists(config_file):
        reader = PyExpatReader()
        dom = reader.fromUri("file:%s" % os.path.abspath(config_file))
        root = dom.documentElement
        nodes = xpath.Evaluate('config/instance[@name="%s"]' % name, dom)
        if len(nodes) > 0:
            print "Instance %s already exist" % name
        else:
            new = dom.createElement('instance')
            new.setAttributeNS('','name', name)
            new.setAttributeNS('','service', service)
            new.setAttributeNS('','description', 'Instance created by script')
            root.insertBefore(new, root.firstChild)
            file = open(config_file, "w")
            UltraPrettyPrint(root, stream=file)
            file.close()        
    else:
        print "Configuration file %s not found. Verify if the local configuration is used" % config_file
        return 1
    return 0

    
def removeInstanceFromConfig(name):
    config_file = "%s/../glite-file-transfer-agents.cfg.xml" % glib.getScriptPath()
    if os.path.exists(config_file):
        reader = PyExpatReader()
        dom = reader.fromUri("file:%s" % os.path.abspath(config_file))
        root = dom.documentElement
        nodes = xpath.Evaluate('config/instance[@name="%s"]' % name, dom)
        if len(nodes) == 1:
            root.removeChild(nodes[0])
            file = open(config_file, "w")
            UltraPrettyPrint(root, stream=file)
            file.close()        
            
        elif len(nodes) == 0:
            print "Instance %s cannot be found"
            return 1
        else:
            print "Multiple instances named %s exist. Cannot remove"
            return 1
    else:
        print "Configuration file %s not found. Verify if the local configuration is used" % config_file
        return 1
    return 0
    
def setDefaults():
    params['transfer-agent-dao-oracle.User'] = params['transfer-agent.DBUser']
    params['transfer-agent-dao-oracle.Password'] = params['transfer-agent.DBPassword']
    if params['file-transfer.DBBackend'] == 'oracle':
        if params['transfer-agent.DBPort'] == '':
            params['transfer-agent.DBPort'] = '1521'
        if params['transfer-agent-dao-oracle.ConnectString'] == '':
            #params['transfer-agent-dao-oracle.ConnectString'] = "${transfer-agent.DBHost}:${transfer-agent.DBPort}/${transfer-agent.DBService}"
            params['transfer-agent-dao-oracle.ConnectString'] = "%s:%s/%s" \
                %(params['transfer-agent.DBHost'], params['transfer-agent.DBPort'], params['transfer-agent.DBService'])
    else:
        params['transfer-agent-dao-mysql.HostName'] = params['transfer-agent.DBHost']
        params['transfer-agent-dao-mysql.DBName'] = params['transfer-agent.DBService']
        params['transfer-agent-dao-mysql.User'] = params['transfer-agent.DBUser']
        params['transfer-agent-dao-mysql.Password'] = params['transfer-agent.DBPassword']
        
        if params['transfer-agent.DBPort'] == '':
            params['transfer-agent.DBPort'] = '3306'
                
#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # The script must be run as root
    if not os.geteuid() == 0:
        print '"\nThis script must be run as root\n'
        sys.exit(1)
        
    # Get an instance of the ConfigParams class
    params = ConfigParams()
    
    # Get an instance of the library class
    glib = gLib()
    glib.verbose = 0
    
    # Load parameters
    loadDefaults(params)
    try:
        opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
        for o, a in opts:
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    except getopt.GetoptError:
        pass
    if glib.loadConfiguration(["%s/../glite-file-transfer-agents.cfg.xml" % glib.getScriptPath(), \
            "%s/../glite-service-discovery.cfg.xml" % glib.getScriptPath(),
            "%s/../glite-rgma-servicetool.cfg.xml" % glib.getScriptPath()], params):
        print "An error occurred while configuring the agents"
        sys.exit(1)
    
    setDefaults()
    verbose = 0
    if params.has_key('glite.installer.verbose'):
        if params['glite.installer.verbose'] == "true":
            verbose = 1
    gLib.verbose = verbose
    
    # Set up the environment
    set_env()
          
    # Instantiate the service classes
    service = glite_data_transfer_agents(params)
    service.verbose = verbose

    # Instantiate the servicediscovery class
    serviceDiscovery = ServiceDiscovery()
    serviceDiscovery.verbose = verbose
    
    # Instantiate the rgma servicetool class
    if not params.has_key('rgma.servicetool.activate'):
        params['rgma.servicetool.activate'] = 'true'
    if params['rgma.servicetool.activate'] == 'true':
        rgmaServicetool = gliteRgmaServicetool()
        rgmaServicetool.verbose = verbose

    # Command line opts if any
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version', \
            'configure', 'remove', 'start', 'stop', 'status', 'siteconfig=', 'instance', 'restart'])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    if len(opts) == 0:
        service.usage()
        sys.exit(0)
    
    # Check cli options
    for o, a in opts:

        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)

        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)

        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            sys.exit(0)
        if o == "--instance":
            import xml.dom.minidom
            from xml.dom.minidom import parse
            from xml import xpath
            from Ft.Lib.pDomletteReader import PyExpatReader 
 
            command = args[0]
            name = args[1]
            if command == 'add':
                print "\n\nAdding instance %s             " % name
                if addInstanceToConfig(name, args[2]) == 0:
                    glib.printOkMessage()
                else:
                    glib.printFailedMessage()
                    sys.exit(1)
                    
                sys.exit(0)
            if command == 'configure':
                print "\n\nConfiguring instance %s             " % name
                name = findFullInstanceName(name)
                if service.configureInstance(name) == 0:
                    glib.printOkMessage()
                else:
                    glib.printFailedMessage()
                    sys.exit(1)
                    
                sys.exit(0)
            if command == 'remove':
                name = findFullInstanceName(name)
                # stop service 
                print "\n\nStopping instance %s             " % name
                if service.stopInstance(name) == 1:
                    glib.printOkMessage()
                else:
                    glib.printFailedMessage()
                    
                # remove configuration files
                instanceParams = {}
                instanceParams = glib.getInstanceParams(name)
   
                # instanceName = name.split(':')[1]
                instanceType, instanceName = name.split(':')
           
                outputBase = '%s/etc/glite-%s-%s' % (os.environ['GLITE_LOCATION'],\
                    instanceType[0:instanceType.rfind('-')], instanceName)
                output = '%s.properties.xml' % outputBase
                log = '%s.log-properties' % outputBase
                
                if os.path.exists(output):
                    print "\n\nRemoving configuration files           "
                    os.remove(output)
                    glib.printOkMessage()
                if os.path.exists(log):
                    print "\n\nRemoving log files           "
                    os.remove(log)
                    glib.printOkMessage()
                # remove instance from configuration file 
                removeInstanceFromConfig(instanceName)
                sys.exit(0)

            if command == 'start':
                print "\n\nStarting instance %s             " % name
                name = findFullInstanceName(name)
                if service.startInstance(name) == 0:
                    glib.printOkMessage()
                else:
                    glib.printFailedMessage()
                    print "Some of instances did not started. Please \
                    consult the script output for detailed information"
                    sys.exit(1)

                sys.exit(0)

            if command == 'stop':
                print "\n\nStopping instance %s             " % name
                name = findFullInstanceName(name)
                if service.stopInstance(name) == 0:
                    glib.printOkMessage()
                else:
                    glib.printFailedMessage()
                    sys.exit(1)
                    
                sys.exit(0)

            if command == 'status':
                name = findFullInstanceName(name)
                sys.exit(service.statusInstance(name))
                                
            print "Command %s does not exist" % command
            sys.exit(1)
            
        if o == "--configure":
            # Check certificates
            if params.has_key('glite.installer.checkcerts'):
                if params['glite.installer.checkcerts'] == "true":
                    if glib.check_certs(params) != 0:
                        print "An error occurred while configuring the %s" \
                            % service.friendly_name
                        sys.exit(1)
            
            service.copyright()
            service.showVersion()
            service.banner()
                
            # Display the parameters if verbose output
            if verbose:
                glib.print_params(params)
                            
            # Configure the service
            return_value = service.configure()
            
            if return_value == 0:

                glib.registerService()

                print "\n\nThe %s configuration was successfully completed\n" % service.friendly_name
                print "You can now start the agents using the --start option of this script\n\n"
    
                sys.exit(0)

            elif return_value == 2:

                print "\n\nThe %s configuration was completed, but one or more warnings" % service.friendly_name
                print "were issued. Please revise them and re-run the script"
                print "or configure and start the %s manually\n" % service.friendly_name

                sys.exit(2)

            else:
                print "\n\nAn unrecoverable error occurred while configuring the %s" \
                    % service.friendly_name

                sys.exit(1)
            
        if o == "--restart":
            instances = glib.getInstances("transfer-")
            try:
                if len(instances) == 0:
                        glib.printInfoMessage("No instance found")
                        sys.exit(1)
                else:
                    for name in instances.keys():
                        instanceParams = {}
                        instanceParams = glib.getInstanceParams(name)
                        type = name.split(':')[0]
                        instanceType = type[type.find('-') + 1:type.rfind('-')]
                        instanceName = name.split(':')[1]
                        agentProcess = 'glite-transfer-%s-%s' % (instanceType, instanceName)
                        startAgentCommand = "/bin/su -l %s -c '%s/bin/glite_data_config_service %s restart > /dev/null 2>&1'" \
                            %(instanceParams['transfer-agent.username'], os.environ['GLITE_LOCATION'], agentProcess)
                        ret = os.system(startAgentCommand)
            except ValueError, msg:
                glib.printErrorMessage("%s - Cannot continue" % msg)
                sys.exit(1)
            sys.exit(ret)
            
        if o == "--start":
            if service.start() == 0:
                print "\n\nThe %s were successfully started\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nOne or more instances of %s did not start correctly. For more\
                information consult the output of the script." % service.friendly_name
                sys.exit(1)

        if o == "--stop":
            if service.stop() == 0:
                print "\n\nThe %s were successfully stopped\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nOne or more instances of %s did not stop correctly. For more\
                information consult the output of the script." % service.friendly_name
                sys.exit(1)

        if o == "--status":
            sys.exit(service.status())

#        if o == "--instancestatus" :

        if o == "--remove":
            glib.printInfoMessage("\n\nStopping all running agents...")
            if service.stop() != 0:
                sys.exit(1)

            retVal = service.remove()
            print "\n\nAll agent instances were successfully removed\n"
            sys.exit(retVal)
